#!/usr/bin/env perl
#
# NAME:
#	ip2bc - display correct broadcast address for an address
#
# SYNOPSIS:
#	ip2bc [options]["file" ...]
#
# DESCRIPTION:
#	This script reads addresses from "file" or stdin.
#	It displays the address, the netmask and broadcast address in
#	a manner suitable for ifconfig(8).
#
#	Options:
#
#	-b "bits"
#		Indicate whether broadcast uses 1's or 0's in the host
#		number. The default is 1.
#
#	-m "mask"
#		The "mask" used to determine the network and host
#		parts of an address.  Can be given hex or dotted
#		decimal.
#		
#	For example:
#.nf
#
#	echo 192.168.168.1 | ip2bc.pl -b1 -m255.255.255.192
#	192.168.168.1 netmask 255.255.255.192 broadcast 192.168.168.63
#.fi
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#	

# RCSid:
#	$Id: ip2bc.pl,v 1.7 2008/01/26 04:25:46 sjg Exp $
#
#	@(#) Copyright (c) 1993 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

if ($0 =~ m,^(.*)/([^/]+)$,) {
  $Mydir = $1;
  $Myname = $2;
} else {
  $Mydir = '.';
  $Myname = $0;
}

require "$Mydir/getopts.pl";

push(@INC, $Mydir);

require 'ip_subs.pl';
require 'dofile.pl';

$opt_m = '0xffffff00';
$opt_b = 1;

&do_file(".${Myname}rc",@RcPath);

&Getopts('m:b:a:');

&main;
exit 0;

sub main {
  if ($opt_a ne '') {
    $cmask = &net2class_mask($opt_a);
    $mask = &net2mask($opt_a);
    $net = &ip2hex($opt_a);
    $b0 = $net & $cmask;
    $b1 = $b0 | ~$mask;
    $n = $b1 ^ $b0;
    
    $max = $mask ^ $cmask;

    printf "ip=$opt_a, mask=%#x, cmask=%#x, max=%#x(%d), n=%#x(%d)\n", $mask, $cmask, $max, $max, $n, $n;

    $Net = $b0;
    for ($i = 0, $j = 0; $j+$i <= $max; $i++, $j += $n) {
      $ip = &hex2ip($Net + $i + $j);
      print "net $i";
      &showbc($ip);
    }
  } else {
    while (<>) {
      chop;
      &showbc($_);
    }
  }
}

sub showbc {
  local($ip) = @_;
  
  $mask = &net2mask($ip);
  $net = &ip2hex($ip);
  $b0 = $net & $mask;
  $b1 = $b0 | ~$mask;
  $b = ($opt_b > 0) ? $b1 : $b0;

  printf "\t%s netmask %s broadcast %s\n",
	  $ip, &hex2ip($mask), &hex2ip($b);
}
